#ifndef NET_IPV4_H
#define NET_IPV4_H

#include <types.h>
#include "net_def.h"

#pragma pack(push, 1)
struct ipv4_head_t {
    struct {
#define IPV4_VERSION    4
        uint8 ihl     : 4;
        uint8 version : 4;
    };
    struct {
        uint8 ecn  : 2;
        uint8 dscp : 6;
    };
    uint16 total_len;
    uint16 ident;
    struct {
#define IPV4_FLAG_RESERVED   (1 << 0)
#define IPV4_FLAG_DF         (1 << 1)
#define IPV4_FLAG_MF         (1 << 2)
        uint16 f_offset_msb : 5;
        uint16 flags        : 3;
        uint16 f_offset_lsb : 8;
    };
    uint8  ttl;
#define IPV4_PROTOCOL_ICMP   0x01
#define IPV4_PROTOCOL_UDP    0x11
#define IPV4_PROTOCOL_TCP    0x06
    uint8  protocol;
    uint16 head_cs;
    uint32 src_ip;
    uint32 dst_ip;
};

struct ipv4_packet_t {
    struct ipv4_head_t head;
    union {
#define IPV4_MAX_OPTIONS_COUNT 4 /* XXX */
        uint32 options[IPV4_MAX_OPTIONS_COUNT];
#define IPV4_MAX_PAYLOAD_SIZE   1536 /* XXX */
        uint8 payload[IPV4_MAX_PAYLOAD_SIZE];
    };
};
#pragma pack(pop)

#define IPV4_HEAD_LEN(ipv4) \
    (ipv4->head.ihl * 4)

int ipv4_process(struct net_eth_frame_t *inf, int flen);
uint8* ipv4_payload(struct ipv4_packet_t *ipv4);
uint16 ipv4_head_cs(struct ipv4_packet_t *ipv4);

#endif

