#ifndef _TCP_H_
#define _TCP_H_

#include <types.h>
#include "ipv4.h"

#pragma pack(push, 1)
struct tcp_packet_head_t {
    uint16 src_port;
    uint16 dst_port;
    uint32 seqn;
    uint32 ackn;
    union {
        struct {
            uint8 ns       : 1;
            uint8 reserved : 3;
            uint8 doffset  : 4;
            uint8 fin      : 1;
            uint8 syn      : 1;
            uint8 rst      : 1;
            uint8 psh      : 1;
            uint8 ack      : 1;
            uint8 urg      : 1;
            uint8 ece      : 1;
            uint8 cwr      : 1;
            uint16 wsize;
        };
        uint32 w3;
    };
    uint16 cs;
    uint16 urgpointer;
};

struct tcp_packet_t {
    struct tcp_packet_head_t head;
    union {
#define OPTION_MAX_LENGTH   40
#define PAYLOAD_MAX_LENGTH  (sizeof(struct net_eth_frame_t) - sizeof(struct tcp_packet_head_t)) /* XXX ipv4 head not considered */
        uint8 options[OPTION_MAX_LENGTH];
        uint8 payload[PAYLOAD_MAX_LENGTH];
    };
};


#pragma pack(pop)

#define TCP_BUF_SIZE    (4 * 1024)
struct tcp_buf_t {
    uint8 buf[TCP_BUF_SIZE];
};



/* transmission control block */
struct tcp_tcb_t {
    BASE_TYPE mutex;

    uint32 src_ip;
    uint32 dst_ip;

    uint16 src_port;
    uint16 dst_port;

    uint16 local_mss;
    uint16 remote_mss;
    uint32 wsize;

    /* send sequence variables */
    uint32 snd_seq;

    /* receive sequence variables */
    uint32 rcv_seq;

    struct tcp_buf_t snd_buf;
    struct tcp_buf_t rcv_buf;

//    /* send sequence variables */
//    uint32 snd_una; /* send unacknowledged */
//    uint32 snd_nxt; /* send next */
//    uint32 snd_wnd; /* send window */
//    uint32 snd_up;  /* send urgent pointer */
//    uint32 snd_wl1; /* segment sequence number used for last window update */
//    uint32 snd_wl2; /* segment ack number used for last window update */
//    uint32 iss;     /* initial send sequence number */
//
//    /* receive sequence variables */
//    uint32 rcv_nxt; /* receive next */
//    uint32 rcv_wnd; /* receive window */
//    uint32 rcv_ug;  /* receive urgent pointer */
//    uint32 irs;     /* initial receive sequence number */
//
//    /* current segment */
//    uint32 seg_seq; /* segment sequence number */
//    uint32 seg_ack; /* segment acknowledgment number */
//    uint32 seg_len; /* segment length */
//    uint32 seg_wnd; /* segment window */
//    uint32 seg_up ; /* segment urgent pointer */
//    uint32 seg_prc; /* segment precedence value */

    enum {
        TCP_STATE_CLOSED = 0,
//        TCP_STATE_LISTEN,
        TCP_STATE_SYN_SENT,
//        TCP_STATE_SYN_RECEIVED,
//        TCP_STATE_ESTABLISHED,
//        TCP_STATE_FIN_WAIT_1,
//        TCP_STATE_FIN_WAIT_2,
//        TCP_STATE_CLOSE_WAIT,
//        TCP_STATE_CLOSING,
//        TCP_STATE_LAST_ACK,
//        TCP_STATE_TIME_WAIT,
    } state;
};

void tcp_init();
void tcp_process(struct ipv4_packet_t *ipv4);
struct tcp_tcb_t * tcp_open();

#endif

