#include <debug.h>
#include <string.h>
#include "net.h"
#include "net_def.h"
#include "udp.h"

//#define DEBUG_UDP

/* NOTE */
#ifndef DEBUG_NET
    #undef DEBUG_UDP
#endif

#ifdef DEBUG_UDP
    #define DPRINT(fmt, ...) dprint(fmt, __VA_ARGS__)
#else
    #define DPRINT(fmt, ...)
#endif

/*
 * ARGS
 *    packet    input packet
 *
 * RETURN
 *     count of bytes in response (size of udp packet), 0 if there is no response
 */
int udp_process(struct ipv4_packet_t *ipv4)
{
    struct udp_packet_t *udp;

    DEBUG_IMSG("=== udp process ===");

    if (ipv4->head.ihl > 5)
        udp = (struct udp_packet_t *)&ipv4->payload[
            (ipv4->head.ihl * 4) - sizeof(struct ipv4_head_t)];
    else
        udp = (struct udp_packet_t *)ipv4->payload;

#ifdef DEBUG_UDP
    {
        DPRINT("<src port = >2xn", udp->head.src_port);
        DPRINT("<dst port = >2xn", udp->head.dst_port);
        DPRINT("<length   = >2xn", udp->head.length);
        DPRINT("<cs       = >2xn", udp->head.cs);
    }
#endif

    /* XXX discard packet longer then 1008 bytes */
    if (NET_HTONS(udp->head.length) > 1008)
        return 0;

//    /* check CS if it not zero */
//    if (udp->head.cs)
//    {
//        if (udp_cs(ipv4) != udp->head.cs)
//        {
//            DEBUG_WMSG("checksum mismatch");
//            return 0;
//        }
//    }

    net_udp_receive(
            NET_HTONL(ipv4->head.src_ip),
            NET_HTONL(ipv4->head.dst_ip),
            NET_HTONS(udp->head.src_port),
            NET_HTONS(udp->head.dst_port),
            udp->payload,
            NET_HTONS(udp->head.length) - sizeof(struct udp_packet_head_t));

//    {
//        uint16 len;
//        uint16 port;
//#define BUF_SIZE 3
//        uint8 buf[BUF_SIZE];
//
//        buf[0] = 0x00;
//        buf[1] = 0xab;
//        buf[2] = 0xac;
//
//        len = (sizeof(struct udp_packet_head_t) + BUF_SIZE);
//        memcpy(udp->payload, buf, BUF_SIZE);
//
////        len = (sizeof(struct udp_packet_head_t) + 1000);
//
//        port               = udp->head.src_port;
//        udp->head.src_port = udp->head.dst_port;
//        udp->head.dst_port = port;
//
//        ipv4->head.total_len = NET_HTONS(IPV4_HEAD_LEN(ipv4) + len);
//
//        udp->head.length   = NET_HTONS(len);
//
//
////        ipv4->head.total_len = NET_HTONS(len);
//        udp->head.cs       = udp_cs(ipv4);
//
//        return len;
//    }

    return 0;
}

/*
 *
 */
uint16 udp_cs(struct ipv4_packet_t *ipv4)
{
    uint32 cs;
    uint32 dlen;
    uint16 *data;

    struct udp_packet_t *udp;
    if (ipv4->head.ihl > 5)
        udp = (struct udp_packet_t *)&ipv4->payload[
            IPV4_HEAD_LEN(ipv4) - sizeof(struct ipv4_head_t)];
    else
        udp = (struct udp_packet_t *)ipv4->payload;

    dlen = (NET_HTONS(ipv4->head.total_len) - IPV4_HEAD_LEN(ipv4));

    cs = (ipv4->head.src_ip & 0xffff) +
         (ipv4->head.src_ip >> 16) +
         (ipv4->head.dst_ip & 0xffff) +
         (ipv4->head.dst_ip >> 16) +
         (ipv4->head.protocol << 8) +
         NET_HTONS(dlen) +
         udp->head.src_port + 
         udp->head.dst_port +
         udp->head.length;

    dlen -= sizeof(struct udp_packet_head_t);
    data = (uint16*)udp->payload;

    while (dlen > 1)
    {
        cs += *data++;
        dlen -= 2;
    }
    if (dlen)
        cs += (*data) & 0xff;

    cs = ~(((cs >> 16) + cs) & 0xffff);
    if (cs == 0)
        cs = 0xffff;

    return cs;
}

