/* 
 *     This file is part of K11, hardware multimedia player.
 * 
 * Copyright (C) 2014 Dmitry Kobylin
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef NVRAM_H
#define NVRAM_H

#include <types.h>

#pragma pack(push, 1)

struct nvram_store_t {
    union {
        struct {
            uint32 dport;       /* reserved for DPort */
            uint16 artist_hash; /* hash of name of last used artist */
            uint16 album_hash;  /* hash of name of last used album  */
            uint16 track_hash;  /* hash of name of last playback track */
            uint16 time;        /* time in second of track */
            uint32 fposition;   /* position in file of playback track */
            uint8  volume;      /* volume level */
            struct {
                uint8 repeat   : 2; /* repeat mode */
                uint8 pause    : 1; /* paused state */
                uint8 resbits  : 5;
            };
            struct {
                uint8 bass   : 4; /* bass enhancement */
                uint8 treble : 4; /* treble enchancement */
            };
            uint8 reserved0;
        };
        struct {
            uint32 reg[5];
        };
    };
};

#pragma pack(pop)

void nvram_load();
void nvram_save();

extern struct nvram_store_t nvram;

#endif

