#include <LPC177x_8x.h>
#include <LPC177x_8x_bits.h>
//#include <clk_cfg.h>
#include <os.h>
#include <debug.h>
#include "suspend.h"
#include "gs/gs.h"
#include "gpioirq.h"
#include "buttons.h"
#include "sdcard/sdcard.h"
#include "vs1053b/vs1053b.h"
#include "eth.h"

static BASE_TYPE suspevent;

static inline void suspend_portable();
static inline void suspend_nonportable();

#define SUSPEND_TIMEOUT (2 * 60 * 1000)

/*
 *
 */
void suspend_task()
{
    while (1)
    {
        if (os_event_wait(&suspevent, (1 << 0),
                OS_FLAG_CLEAR, OS_MS2TICK(SUSPEND_TIMEOUT)) == OS_ERR_NONE)
            continue;

        dprint("sn", "Suspend...");

        os_sched_lock();
        suspend_portable();

        suspend_nonportable();
        dprint("sn", "NOTREACHED");
    }
}

/*
 * defer suspend
 */
void suspend_defer()
{
    os_event_raise_ns(&suspevent, (1 << 0));
}

/*
 *
 */
static inline void suspend_portable()
{
    eth_suspend();
    gs_suspend();
    sdcard_suspend();
    vs1053b_suspend();
}

/*
 *
 */
static inline void suspend_nonportable()
{
    /* disable interrupts */
    asm volatile ("cpsid i \n");

    /* TODO disable pull-ups on IO */

    {
        int i;

        /* disable SysTick */
        SysTick->CTRL = 0;

        /* disable all interrupts */
        for (i = 0; i < 4 /* XXX */; i++)
        {
            NVIC->ICER[i] = 0xffffffff;
            NVIC->ICPR[i] = 0xffffffff;
        }

        /* enable only buttons interrupt */
        NVIC_EnableIRQ(GPIO_IRQn);
    }

//    /* XXX */
//    LPC_EMC->DynamicControl = 0x03 | (1 << 13);

    /* 
     * Disable all PCONP except GPIO and RTC
     */
    LPC_SC->PCONP = PCONP_PCGPIO | PCONP_PCRTC;

    /*
     * Change clock sources, select suspend mode
     */
    {
#define CLKSRCSEL_INTRC    (0x00)
        LPC_SC->CLKSRCSEL = CLKSRCSEL_INTRC;
#define CCLKSEL_CCLKSEL_SYSCLK (0 << 8)
#define CCLKSEL_CCLKSEL_PLL    (1 << 8)
#define CCLKSEL_CCLKDIV(val)   val

        LPC_SC->CCLKSEL   = CCLKSEL_CCLKSEL_SYSCLK | CCLKSEL_CCLKDIV(1);
        LPC_SC->USBCLKSEL = 0;

        SCB->SCR = SCB_SCR_SLEEPDEEP_Msk | SCB_SCR_SLEEPONEXIT_Msk;
        LPC_SC->PCLKSEL = 31;
        LPC_SC->SCS     = 0;

        LPC_SC->PLL0CON = 0;
        LPC_SC->PLL1CON = 0;

#define PCON_PM_SLEEP            (0x00 << 0)
#define PCON_PM_POWER_DOWN       (0x01 << 0)
#define PCON_PM_DEEP_POWER_DOWN  (0x03 << 0)

#define PCON_BODRPM     (1 << 2)
#define PCON_BOGD       (1 << 3)
#define PCON_BORD       (1 << 4)
#define PCON_SMFLAG     (1 << 8)
#define PCON_DSFLAG     (1 << 9)
#define PCON_PDFLAG     (1 << 10)
#define PCON_DPDFLAG    (1 << 11)
        LPC_SC->PCON = 
            PCON_BODRPM   |
            PCON_BOGD     |
            PCON_BORD     |
            PCON_PM_POWER_DOWN
            ;
    }

    /*
     * Move to suspend state
     */
    asm volatile(
            "dsb     \n"
            "wfi     \n"
            );

    /* NOTE SDRAM not operational anymore, be care */

    /* XXX check for spurious wake up events */

    /*
     * Reset after resume
     */
    SCB->AIRCR = (0x05fa << SCB_AIRCR_VECTKEY_Pos) | SCB_AIRCR_SYSRESETREQ_Msk;
    asm volatile("dsb");
}

