
lappend auto_path [pwd]
package require dport

namespace import ::dport::util::crc16_ccitt
namespace import ::dport::util::num2bin
namespace import ::dport::util::bin2num
namespace import ::dport::util::bin2list

#
# reset device.
#
#
proc devreset {} {
    #
    # NOTE 
    #     500 ms timeout on device, so wait a little longer
    #
    reset
    after 800
}

#
# reset device and stop device in bootloader (debug handler)
#
proc wboot {value} {
    set RTC_GPREG     0x40024044 

    # write magic number to RTC memory
    writemem $RTC_GPREG [num2bin $value 4]

    # reset device
    devreset

    # check that we actualy in right point of program
    set bkpt [bin2num [readmem [expr {[readreg PC] - 2}] 2] 0 2]
    puts "bkpt = [format %04X $bkpt]"
    if {$bkpt != 0xbeab} {
        puts stderr "bkpt instruciton mismatch, load failed"
        return 0
    }

    return 1
}

#
# write image to FLASH
#
proc wf {} {
    set image main.bin

    if {![wboot 0x4444BEEF]} {
        return
    }

    # get length
    set length [file size $image]

    # calculate crc
    set fd [open $image r] 
    fconfigure $fd -translation binary
    set crc [crc16_ccitt 0xffff [bin2list [read $fd]]]
    close $fd

    puts "LENGTH  [format %08X $length]"
    puts "CRC     [format %04X $crc]"

    writeflash 0x8000 [num2bin $length 4][num2bin $crc 2]
    writeimage main.bin 0x9000

    # continue execution
    cont
}

#
# write image to SDRAM
#
proc wr {} {
    set image main.bin
    set SDRAM_OFFSET  0xa0000000

    if {![wboot 0x4444DEAD]} {
        return
    }

    # write file data to SDRAM
    set fd [open $image r] 
    fconfigure $fd -translation binary
    writemem $SDRAM_OFFSET [read $fd]
    close $fd

#    puts "dump memory"
#
#    set fd [open /dev/shm/mem.dat w]
#    fconfigure $fd -translation binary
#    puts -nonewline $fd [readmem $SDRAM_OFFSET [file size $image]]
#    close $fd
#
#    puts "done"

    # continue execution
    cont
}

#
# print tasks state
#
proc pt {} {
    userf 0
}

#
# print tasks stack level
#
proc pl {} {
    userf 1
}

#quit ; # exit from program

